# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys

from ddt import data
from ddt import ddt
from subunit import RemotedTestCase
from testtools import PlaceHolder

from os_testr import subunit2html
from os_testr.tests import base


@ddt
class TestSubunit2html(base.TestCase):
    @data(RemotedTestCase, PlaceHolder)
    def test_class_parsing(self, test_cls):
        """Tests that the class paths are parsed for v1 & v2 tests"""
        test_ = test_cls("example.path.to.test.method")
        obj_ = subunit2html.HtmlOutput()
        cls_ = []
        obj_._add_cls({}, cls_, test_, ())
        self.assertEqual("example.path.to.test", cls_[0].name)

    @data(RemotedTestCase, PlaceHolder)
    def test_result_sorting(self, test_cls):
        tests = []
        for i in range(9):
            tests.append(test_cls('example.path.to.test%d.method' % i))
        # addFailure, addError, and addSkip need the real exc_info
        try:
            raise Exception('fake')
        except Exception:
            err = sys.exc_info()
        obj = subunit2html.HtmlOutput()
        obj.addSuccess(tests[3])
        obj.addSuccess(tests[1])
        # example.path.to.test2 has a failure
        obj.addFailure(tests[2], err)
        obj.addSkip(tests[0], err)
        obj.addSuccess(tests[8])
        # example.path.to.test5 has a failure (error)
        obj.addError(tests[5], err)
        # example.path.to.test4 has a failure
        obj.addFailure(tests[4], err)
        obj.addSuccess(tests[7])
        # example.path.to.test6 has a success, a failure, and a success
        obj.addSuccess(tests[6])
        obj.addFailure(tests[6], err)
        obj.addSuccess(tests[6])
        sorted_result = obj._sortResult(obj.result)
        # _sortResult returns a list of results of format:
        #   [(class, [test_result_tuple, ...]), ...]
        # sorted by str(class)
        #
        # Classes with failures (2, 4, 5, and 6) should be sorted separately
        # at the top. The rest of the classes should be in sorted order after.
        expected_class_order = ['example.path.to.test2',
                                'example.path.to.test4',
                                'example.path.to.test5',
                                'example.path.to.test6',
                                'example.path.to.test0',
                                'example.path.to.test1',
                                'example.path.to.test3',
                                'example.path.to.test7',
                                'example.path.to.test8']
        for i, r in enumerate(sorted_result):
            self.assertEqual(expected_class_order[i], str(r[0]))

    @data(RemotedTestCase, PlaceHolder)
    def test_generate_report_with_no_ascii_characters(self, test_cls):
        # The test examines a case where an error containing no ascii
        # characters is received.
        test = test_cls(u'example.path.to.test1.method')
        try:
            raise Exception('\xe2\x82\xa5')
        except Exception:
            err = sys.exc_info()
        obj = subunit2html.HtmlOutput()
        # Add failure that contains no ascii characters
        obj.addFailure(test, err)
        obj._generate_report()



# Copyright 2015 SUSE Linux GmbH
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from datetime import datetime as dt
import io
import os
import subprocess
import sys
from unittest.mock import patch

from ddt import data
from ddt import ddt
from ddt import unpack
import six

from os_testr import subunit_trace
from os_testr.tests import base


@ddt
class TestSubunitTrace(base.TestCase):

    @data(([dt(2015, 4, 17, 22, 23, 14, 111111),
            dt(2015, 4, 17, 22, 23, 14, 111111)],
           "0.000000s"),
          ([dt(2015, 4, 17, 22, 23, 14, 111111),
            dt(2015, 4, 17, 22, 23, 15, 111111)],
           "1.000000s"),
          ([dt(2015, 4, 17, 22, 23, 14, 111111),
            None],
           ""))
    @unpack
    def test_get_durating(self, timestamps, expected_result):
        self.assertEqual(subunit_trace.get_duration(timestamps),
                         expected_result)

    @data(([dt(2015, 4, 17, 22, 23, 14, 111111),
            dt(2015, 4, 17, 22, 23, 14, 111111)],
           0.0),
          ([dt(2015, 4, 17, 22, 23, 14, 111111),
            dt(2015, 4, 17, 22, 23, 15, 111111)],
           1.0),
          ([dt(2015, 4, 17, 22, 23, 14, 111111),
            None],
           0.0))
    @unpack
    def test_run_time(self, timestamps, expected_result):
        patched_res = {
            0: [
                {'timestamps': timestamps}
            ]
        }
        with patch.dict(subunit_trace.RESULTS, patched_res, clear=True):
            self.assertEqual(subunit_trace.run_time(), expected_result)

    def test_return_code_all_skips(self):
        skips_stream = os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            'sample_streams/all_skips.subunit')
        p = subprocess.Popen(['subunit-trace'], stdin=subprocess.PIPE)
        with open(skips_stream, 'rb') as stream:
            p.communicate(stream.read())
        self.assertEqual(1, p.returncode)

    def test_return_code_normal_run(self):
        regular_stream = os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            'sample_streams/successful.subunit')
        p = subprocess.Popen(['subunit-trace'], stdin=subprocess.PIPE)
        with open(regular_stream, 'rb') as stream:
            p.communicate(stream.read())
        self.assertEqual(0, p.returncode)

    def test_trace(self):
        regular_stream = os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            'sample_streams/successful.subunit')
        bytes_ = io.BytesIO()
        with open(regular_stream, 'rb') as stream:
            bytes_.write(six.binary_type(stream.read()))
        bytes_.seek(0)
        stdin = io.TextIOWrapper(io.BufferedReader(bytes_))
        returncode = subunit_trace.trace(stdin, sys.stdout)
        self.assertEqual(0, returncode)



# Copyright 2016 Hewlett Packard Enterprise Development LP
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import six
import sys

from ddt import data
from ddt import ddt
from ddt import unpack

from os_testr.tests import base
from os_testr.utils import colorizer


@ddt
class TestNullColorizer(base.TestCase):

    @data(None, "foo", sys.stdout, )
    def test_supported_always_true(self, stream):
        self.assertTrue(colorizer.NullColorizer.supported(stream))

    @data(("foo", "red"), ("foo", "bar"))
    @unpack
    def test_write_string_ignore_color(self, text, color):
        output = six.StringIO()
        c = colorizer.NullColorizer(output)
        c.write(text, color)
        self.assertEqual(text, output.getvalue())

    @data((None, "red"), (None, None))
    @unpack
    def test_write_none_exception(self, text, color):
        c = colorizer.NullColorizer(sys.stdout)
        self.assertRaises(TypeError, c.write, text, color)


@ddt
class TestAnsiColorizer(base.TestCase):

    def test_supported_false(self):
        # NOTE(masayukig): This returns False because our unittest env isn't
        #  interactive
        self.assertFalse(colorizer.AnsiColorizer.supported(sys.stdout))

    @data(None, "foo")
    def test_supported_error(self, stream):
        self.assertRaises(AttributeError,
                          colorizer.AnsiColorizer.supported, stream)

    @data(("foo", "red", "31"), ("foo", "blue", "34"))
    @unpack
    def test_write_string_valid_color(self, text, color, color_code):
        output = six.StringIO()
        c = colorizer.AnsiColorizer(output)
        c.write(text, color)
        self.assertIn(text, output.getvalue())
        self.assertIn(color_code, output.getvalue())

    @data(("foo", None), ("foo", "invalid_color"))
    @unpack
    def test_write_string_invalid_color(self, text, color):
        output = six.StringIO()
        c = colorizer.AnsiColorizer(output)
        self.assertRaises(KeyError, c.write, text, color)
